@@ -14,7 +14,6 @@
 #include "vpl/preview/defs.hpp"
 #include "vpl/preview/exception.hpp"
 #include "vpl/preview/extension_buffer_list.hpp"
-#include "vpl/preview/frame_pool.hpp"
 #include "vpl/preview/frame_surface.hpp"
 #include "vpl/preview/future.hpp"
 #include "vpl/preview/impl_selector.hpp"
@@ -67,10 +66,10 @@ protected:
         if (sts != MFX_ERR_NONE) {
             this->selected_impl_ = 0;
         }
-        this->version_ = { 0, 0 };
+        this->version_ = { { 0, 0 } };
         sts            = MFXQueryVersion(this->session_, &this->version_);
         if (sts != MFX_ERR_NONE) {
-            this->version_ = { 0, 0 };
+            this->version_ = { { 0, 0 } };
         }
         init_accelerator_handle();
     }
@@ -88,7 +87,7 @@ public:
     /// @return Implementation capabilities.
     std::shared_ptr<VideoParams> Caps() {
         std::shared_ptr<VideoParams> caps = std::make_shared<VideoParams>();
-        detail::c_api_invoker e(detail::default_checker,
+        [[maybe_unused]] detail::c_api_invoker e(detail::default_checker,
                                 c_api_callable_.query,
                                 session_,
                                 nullptr,
@@ -101,7 +100,7 @@ public:
     /// @return Corrected implementation capabilities.
     std::shared_ptr<VideoParams> Verify(VideoParams *param) {
         std::shared_ptr<VideoParams> out = std::make_shared<VideoParams>();
-        detail::c_api_invoker e(
+        [[maybe_unused]] detail::c_api_invoker e(
             detail::default_checker,
             std::bind(c_api_callable_.query, session_, param->getMfx(), out->getMfx()));
         return out;
@@ -147,7 +146,7 @@ public:
     /// @return Session parameters.
     std::shared_ptr<VideoParams> working_params() {
         std::shared_ptr<VideoParams> out = std::make_shared<VideoParams>();
-        detail::c_api_invoker e(detail::default_checker,
+        [[maybe_unused]] detail::c_api_invoker e(detail::default_checker,
                                 c_api_callable_.params,
                                 session_,
                                 out->getMfx());
@@ -198,23 +197,23 @@ protected:
             return;
 
         #ifdef LIBVA_SUPPORT
-            if ((impl & MFX_IMPL_VIA_VAAPI) == MFX_IMPL_VIA_VAAPI) {
-                VADisplay va_dpy = NULL;
-                // initialize VAAPI context and set session handle (req in Linux)
-                fd_ = open("/dev/dri/renderD128", O_RDWR);
-                if (fd_ >= 0) {
-                    va_dpy = vaGetDisplayDRM(fd_);
-                    if (va_dpy) {
-                        int major_version = 0, minor_version = 0;
-                        if (VA_STATUS_SUCCESS == vaInitialize(va_dpy, &major_version, &minor_version)) {
-                            MFXVideoCORE_SetHandle(session_,
-                                                static_cast<mfxHandleType>(MFX_HANDLE_VA_DISPLAY),
-                                                va_dpy);
-                        }
+        if ((impl & MFX_IMPL_VIA_VAAPI) == MFX_IMPL_VIA_VAAPI) {
+            VADisplay va_dpy = NULL;
+            // initialize VAAPI context and set session handle (req in Linux)
+            fd_ = open("/dev/dri/renderD128", O_RDWR);
+            if (fd_ >= 0) {
+                va_dpy = vaGetDisplayDRM(fd_);
+                if (va_dpy) {
+                    int major_version = 0, minor_version = 0;
+                    if (VA_STATUS_SUCCESS == vaInitialize(va_dpy, &major_version, &minor_version)) {
+                        MFXVideoCORE_SetHandle(session_,
+                                            static_cast<mfxHandleType>(MFX_HANDLE_VA_DISPLAY),
+                                            va_dpy);
                     }
                 }
-                accelerator_handle = va_dpy;
             }
+            accelerator_handle = va_dpy;
+        }
         #endif
     }
 
@@ -289,8 +288,7 @@ public:
             : session(sel, detail::CAPI<>::Decoder),
               bits_(codecID),
               rdr_(rdr),
-              params_(),
-              list_() {
+              params_() {
         component_ = component::decoder;
         params_.set_CodecId(codecID);
         params_.clear_extension_buffers();
@@ -306,45 +304,7 @@ public:
             : session(sel, detail::CAPI<>::Decoder),
               bits_((codec_format_fourcc)params.get_CodecId()),
               rdr_(rdr),
-              params_(params),
-              list_() {
-        component_ = component::decoder;
-        params_.clear_extension_buffers();
-    }
-
-    /// @brief Constructs decoder session
-    /// @param[in] sel Implementation selector
-    /// @param[in] codecID Codec ID
-    /// @param[in] rdr Bitstream reader
-    /// @param[in] list Process list
-    decode_session(const implemetation_selector &sel,
-                   codec_format_fourcc codecID,
-                   Reader *rdr,
-                   const decoder_process_list &list)
-            : session(sel, detail::CAPI<>::Decoder),
-              bits_(codecID),
-              rdr_(rdr),
-              params_(),
-              list_(list) {
-        component_ = component::decoder;
-        params_.set_CodecId(codecID);
-        params_.clear_extension_buffers();
-    }
-
-    /// @brief Constructs decoder session
-    /// @param[in] sel Implementation selector
-    /// @param[in] params Video params
-    /// @param[in] rdr Bitstream reader
-    /// @param[in] list Process list
-    decode_session(const implemetation_selector &sel,
-                   const decoder_video_param &params,
-                   Reader *rdr,
-                   const decoder_process_list &list)
-            : session(sel, detail::CAPI<>::Decoder),
-              bits_((codec_format_fourcc)params.get_CodecId()),
-              rdr_(rdr),
-              params_(params),
-              list_(list) {
+              params_(params) {
         component_ = component::decoder;
         params_.clear_extension_buffers();
     }
@@ -496,7 +456,7 @@ public:
     std::shared_ptr<decode_stat> getStat() {
         std::shared_ptr<decode_stat> out = std::make_shared<decode_stat>();
         decode_stat *dec_stat            = out.get();
-        detail::c_api_invoker e(detail::default_checker,
+        [[maybe_unused]] detail::c_api_invoker e(detail::default_checker,
                                 MFXVideoDECODE_GetDecodeStat,
                                 session_,
                                 dec_stat->get_raw());
@@ -516,8 +476,6 @@ protected:
     Reader *rdr_;
     /// @brief Video params
     decoder_video_param params_;
-    /// @brief Process list
-    decoder_process_list list_;
 };
 
 /// @brief Manages encoder's sessions.
@@ -580,10 +538,33 @@ public:
                         encoder_process_list list = {}) {
         mfxSyncPoint sp;
         mfxFrameSurface1 *surf = in_surface.get() ? in_surface.get()->get_raw_ptr() : nullptr;
+        std::shared_ptr<mfxEncodeCtrl> ctrl = nullptr;
+        bool alocated_ctrl                  = false;
 
         if (nullptr == surf) {
             state_ = state::Draining;
         }
+        if(list.get_size() && list.has_buffer<0>()) {
+            ctrl.reset(list.get_buffer<mfxEncodeCtrl, 0>(), [](mfxEncodeCtrl *p) { p = nullptr; });
+            std::cout << "enc ctrl from the list" << std::endl;
+        } else {
+            ctrl = std::make_shared<mfxEncodeCtrl>();
+            *ctrl = {0};
+            alocated_ctrl = true;
+        }
+        
+        // Asumption: Encoder will copy-in all extension buffers.
+        if (auto [buffers, size] = list.get_raw_ext_buffers(); size) {
+            ctrl->ExtParam = buffers;
+            ctrl->NumExtParam = (mfxU16)size;
+        } else {
+            if (alocated_ctrl) {
+                ctrl.reset();
+            } else {
+                ctrl->ExtParam = 0;
+                ctrl->NumExtParam = 0;
+            }
+        }
         detail::c_api_invoker e({ [](mfxStatus s) {
                                     switch (s) {
                                         case MFX_ERR_MORE_DATA:
@@ -599,7 +580,7 @@ public:
                                 } },
                                 MFXVideoENCODE_EncodeFrameAsync,
                                 session_,
-                                nullptr,
+                                ctrl.get(),
                                 surf,
                                 (*bs.get())(),
                                 &sp);
@@ -743,7 +724,7 @@ public:
     /// @brief Dtor
     ~vpp_session() {}
 
-    /// @brief Allocate and return shared pointer to the surface
+    /// @brief Allocate and return shared pointer to the input surface
     /// @todo temporary method
     /// @return Shared pointer to the allocated surface
     auto alloc_input() {
@@ -756,6 +737,21 @@ public:
         return std::make_shared<frame_surface>(surface);
     }
 
+    /// @brief Allocate internal raw surface and attach it to the output surface
+    /// @param[inout] out_surface Reference to the frame_surface.
+    /// @todo temporary method
+    void alloc_output(std::shared_ptr<frame_surface> &out_surface) {
+        mfxFrameSurface1 *surface = nullptr;
+        detail::c_api_invoker e(detail::default_checker,
+                                MFXMemory_GetSurfaceForVPPOut,
+                                session_,
+                                &surface);
+
+        out_surface->inject(surface, out_surface.use_count() + 1);
+
+        return;
+    }
+
     /// @brief Initializes session with given parameters and extention buffers.
     /// @param[in] par Pointer to the parameters.
     /// @param[in] list List of extention buffers.
@@ -763,14 +759,6 @@ public:
     /// @todo This method brakes RAII concept. Need to move to ctor.
     status Init(vpp_video_param *par, vpp_init_reset_list list = {}) {
         status init_sts                 = session::Init(par, list);
-        mfxFrameAllocRequest request[2] = { 0 };
-
-        detail::c_api_invoker e(detail::default_checker,
-                                MFXVideoVPP_QueryIOSurf,
-                                session_,
-                                par->getMfx(),
-                                request);
-        out_frames_allocator_.attach_frame_info(oneapi::vpl::frame_info(request[1].Info));
         return init_sts;
     }
 
@@ -800,7 +788,7 @@ public:
         if (nullptr == surf) {
             state_ = state::Draining;
         }
-        out_surface = out_frames_allocator_.acquire();
+        alloc_output(out_surface);
         detail::c_api_invoker e({ [](mfxStatus s) {
                                     switch (s) {
                                         case MFX_ERR_MORE_DATA:
@@ -821,10 +809,6 @@ public:
                                 nullptr,
                                 &sp);
 
-        if (out_surface) {
-            out_surface->associate_context(session_, sp);
-        }
-
         if (e.sts_ == MFX_ERR_MORE_DATA && state_ == state::Draining) {
             state_ = state::Done;
             out_surface.reset();
@@ -928,10 +912,6 @@ public:
 protected:
     /// @brief Raw freames reader
     frame_source_reader *rdr_;
-
-    /// @brief Temporal naive surfaces allocator.
-    /// @todo Replace the API 2.1 new functions for the memory allocation
-    temporal_frame_allocator out_frames_allocator_;
 };
 
 } // namespace vpl
