@@ -1,13 +1,17 @@
+/*############################################################################
+  # Copyright (C) 2005 Intel Corporation
+  #
+  # SPDX-License-Identifier: MIT
+  ############################################################################*/
 
-#if (MFX_VERSION < 2000)
-    #include "mfx_samples_config.h"
+#include "mfx_samples_config.h"
 
-    #include "pipeline_user.h"
-    #include "sysmem_allocator.h"
+#include "pipeline_user.h"
+#include "sysmem_allocator.h"
 
-    #ifndef MFX_VERSION
-        #error MFX_VERSION not defined
-    #endif
+#ifndef MFX_VERSION
+    #error MFX_VERSION not defined
+#endif
 
 mfxStatus CUserPipeline::InitRotateParam(sInputParams* pInParams) {
     MSDK_CHECK_POINTER(pInParams, MFX_ERR_NULL_PTR);
@@ -133,11 +137,11 @@ void CUserPipeline::DeleteFrames() {
 CUserPipeline::CUserPipeline() : CEncodingPipeline() {
     m_pPluginSurfaces = NULL;
     m_PluginModule    = NULL;
-    m_pusrPlugin      = NULL;
     MSDK_ZERO_MEMORY(m_PluginResponse);
     MSDK_ZERO_MEMORY(m_pluginVideoParams);
     MSDK_ZERO_MEMORY(m_RotateParams);
-    m_MVCflags = MVC_DISABLED;
+    m_MVCflags       = MVC_DISABLED;
+    m_nSyncOpTimeout = MSDK_WAIT_INTERVAL;
 }
 
 CUserPipeline::~CUserPipeline() {
@@ -147,20 +151,10 @@ CUserPipeline::~CUserPipeline() {
 mfxStatus CUserPipeline::Init(sInputParams* pParams) {
     MSDK_CHECK_POINTER(pParams, MFX_ERR_NULL_PTR);
 
+    m_pLoader.reset(new VPLImplementationLoader);
+
     mfxStatus sts = MFX_ERR_NONE;
 
-    m_PluginModule = msdk_so_load(pParams->strPluginDLLPath);
-    MSDK_CHECK_POINTER(m_PluginModule, MFX_ERR_NOT_FOUND);
-
-    PluginModuleTemplate::fncCreateGenericPlugin pCreateFunc =
-        (PluginModuleTemplate::fncCreateGenericPlugin)msdk_so_get_addr(m_PluginModule,
-                                                                       "mfxCreateGenericPlugin");
-
-    MSDK_CHECK_POINTER(pCreateFunc, MFX_ERR_NOT_FOUND);
-
-    m_pusrPlugin = (*pCreateFunc)();
-    MSDK_CHECK_POINTER(m_pusrPlugin, MFX_ERR_NOT_FOUND);
-
     // prepare input file reader
     sts = m_FileReader.Init(pParams->InputFiles, pParams->FileInputFourCC);
     MSDK_CHECK_STATUS(sts, "m_FileReader.Init failed");
@@ -171,6 +165,8 @@ mfxStatus CUserPipeline::Init(sInputParams* pParams) {
     m_nTimeout   = pParams->nTimeout;
     m_bCutOutput = !pParams->bUncut;
 
+    m_nSyncOpTimeout = pParams->nSyncOpTimeout ? pParams->nSyncOpTimeout : MSDK_WAIT_INTERVAL;
+
     // prepare output file writer
     sts = InitFileWriters(pParams);
     MSDK_CHECK_STATUS(sts, "InitFileWriters failed");
@@ -182,44 +178,28 @@ mfxStatus CUserPipeline::Init(sInputParams* pParams) {
     if (D3D11_MEMORY == pParams->memType)
         impl |= MFX_IMPL_VIA_D3D11;
 
-    mfxVersion min_version;
-    mfxVersion version; // real API version with which library is initialized
+    if (!pParams->accelerationMode && pParams->bUseHWLib) {
+#if D3D_SURFACES_SUPPORT
+        pParams->accelerationMode = MFX_ACCEL_MODE_VIA_D3D11;
+#elif defined(LIBVA_SUPPORT)
+        pParams->accelerationMode = MFX_ACCEL_MODE_VIA_VAAPI;
+#endif
+    }
 
-    // we set version to 1.0 and later we will query actual version of the library which will got leaded
-    min_version.Major = 1;
-    min_version.Minor = 0;
+    sts = m_pLoader->ConfigureImplementation(impl);
+    MSDK_CHECK_STATUS(sts, "m_mfxSession.ConfigureImplementation failed");
+    sts = m_pLoader->ConfigureAccelerationMode(pParams->accelerationMode, impl);
+    MSDK_CHECK_STATUS(sts, "m_mfxSession.ConfigureAccelerationMode failed");
+    sts = m_pLoader->EnumImplementations();
+    MSDK_CHECK_STATUS(sts, "m_mfxSession.EnumImplementations failed");
 
     // create a session for the second vpp and encode
-    sts = m_mfxSession.Init(impl, &min_version);
-    MSDK_CHECK_STATUS(sts, "m_mfxSession.Init failed");
+    sts = m_mfxSession.CreateSession(m_pLoader.get());
+    MSDK_CHECK_STATUS(sts, "m_mfxSession.CreateSession failed");
 
-    sts = MFXQueryVersion(m_mfxSession, &version); // get real API version of the loaded library
-    MSDK_CHECK_STATUS(sts, "MFXQueryVersion failed");
+    mfxVersion version = m_pLoader->GetVersion(); // get real API version of the loaded library
 
     if (CheckVersion(&version, MSDK_FEATURE_PLUGIN_API)) {
-        // we check if codec is distributed as a mediasdk plugin and load it if yes
-        // else if codec is not in the list of mediasdk plugins, we assume, that it is supported inside mediasdk library
-
-        // in case of HW library (-hw key) we will firstly try to load HW plugin
-        // in case of failure - we will try SW one
-        mfxIMPL impl2 = pParams->bUseHWLib ? MFX_IMPL_HARDWARE : MFX_IMPL_SOFTWARE;
-
-        if (AreGuidsEqual(MSDK_PLUGINGUID_NULL, pParams->pluginParams.pluginGuid)) {
-            pParams->pluginParams.pluginGuid =
-                msdkGetPluginUID(impl2, MSDK_VENCODE, pParams->CodecId);
-        }
-        if (AreGuidsEqual(pParams->pluginParams.pluginGuid, MSDK_PLUGINGUID_NULL) &&
-            impl2 == MFX_IMPL_HARDWARE)
-            pParams->pluginParams.pluginGuid =
-                msdkGetPluginUID(MFX_IMPL_SOFTWARE, MSDK_VENCODE, pParams->CodecId);
-        if (!AreGuidsEqual(pParams->pluginParams.pluginGuid, MSDK_PLUGINGUID_NULL)) {
-            m_pPlugin.reset(LoadPlugin(MFX_PLUGINTYPE_VIDEO_ENCODE,
-                                       m_mfxSession,
-                                       pParams->pluginParams.pluginGuid,
-                                       1));
-            if (m_pPlugin.get() == NULL)
-                sts = MFX_ERR_UNSUPPORTED;
-        }
     }
 
     // create encoder
@@ -238,27 +218,12 @@ mfxStatus CUserPipeline::Init(sInputParams* pParams) {
 
     sts = ResetMFXComponents(pParams);
     MSDK_CHECK_STATUS(sts, "ResetMFXComponents failed");
-    // register plugin callbacks in Media SDK
-    mfxPlugin plg = make_mfx_plugin_adapter(m_pusrPlugin);
-    sts           = MFXVideoUSER_Register(m_mfxSession, 0, &plg);
-    MSDK_CHECK_STATUS(sts, "MFXVideoUSER_Register failed");
-
-    // need to call Init after registration because mfxCore interface is needed
-    sts = m_pusrPlugin->Init(&m_pluginVideoParams);
-    MSDK_CHECK_STATUS(sts, "m_pusrPlugin->Init failed");
-
-    sts = m_pusrPlugin->SetAuxParams(&m_RotateParams, sizeof(m_RotateParams));
-    MSDK_CHECK_STATUS(sts, "m_pusrPlugin->SetAuxParams failed");
 
     return MFX_ERR_NONE;
 }
 
 void CUserPipeline::Close() {
-    MFXVideoUSER_Unregister(m_mfxSession, 0);
-
     CEncodingPipeline::Close();
-
-    MSDK_SAFE_DELETE(m_pusrPlugin);
     if (m_PluginModule) {
         msdk_so_free(m_PluginModule);
         m_PluginModule = NULL;
@@ -292,7 +257,10 @@ mfxStatus CUserPipeline::ResetMFXComponents(sInputParams* pParams) {
                           m_FileWriters.first,
                           m_mfxEncParams.AsyncDepth,
                           nEncodedDataBufferSize,
-                          m_FileWriters.second);
+                          pParams->CodecId,
+                          m_FileWriters.second,
+                          pParams->bUseHWLib);
+
     MSDK_CHECK_STATUS(sts, "m_TaskPool.Init failed");
 
     sts = FillBuffers();
@@ -340,22 +308,6 @@ mfxStatus CUserPipeline::Run() {
         nEncSurfIdx = GetFreeSurface(m_pEncSurfaces, m_EncResponse.NumFrameActual);
         MSDK_CHECK_ERROR(nEncSurfIdx, MSDK_INVALID_SURF_IDX, MFX_ERR_MEMORY_ALLOC);
 
-        // rotation
-        for (;;) {
-            mfxHDL h1, h2;
-            h1  = &m_pPluginSurfaces[nRotateSurfIdx];
-            h2  = &m_pEncSurfaces[nEncSurfIdx];
-            sts = MFXVideoUSER_ProcessFrameAsync(m_mfxSession, &h1, 1, &h2, 1, &RotateSyncPoint);
-
-            if (MFX_WRN_DEVICE_BUSY == sts) {
-                MSDK_SLEEP(1); // just wait and then repeat the same call
-            }
-            else {
-                break;
-            }
-        }
-        MSDK_BREAK_ON_ERROR(sts);
-
         // save the id of preceding rotate task which will produce input data for the encode task
         if (RotateSyncPoint) {
             pCurrentTask->DependentVppTasks.push_back(RotateSyncPoint);
@@ -441,7 +393,7 @@ mfxStatus CUserPipeline::Run() {
 
     // synchronize all tasks that are left in task pool
     while (MFX_ERR_NONE == sts) {
-        sts = m_TaskPool.SynchronizeFirstTask();
+        sts = m_TaskPool.SynchronizeFirstTask(m_nSyncOpTimeout);
     }
 
     // MFX_ERR_NOT_FOUND is the correct status to exit the loop with,
@@ -479,4 +431,3 @@ void CUserPipeline::PrintInfo() {
 
     CEncodingPipeline::PrintInfo();
 }
-#endif
